'use strict';


var synopassApp = angular.module('synopassApp',  [],function($httpProvider){

        // Use x-www-form-urlencoded Content-Type
        $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

        // Override $http service's default transformRequest
        $httpProvider.defaults.transformRequest = [function(data)
        {
            /**
             * The workhorse; converts an object to x-www-form-urlencoded serialization.
             * @param {Object} obj
             * @return {String}
             */
            var param = function(obj)
            {
                var query = '';
                var name, value, fullSubName, subName, subValue, innerObj, i;

                for(name in obj)
                {
                    value = obj[name];

                    if(value instanceof Array)
                    {
                        for(i=0; i<value.length; ++i)
                        {
                            subValue = value[i];
                            fullSubName = name + '[' + i + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value instanceof Object)
                    {
                        for(subName in value)
                        {
                            subValue = value[subName];
                            fullSubName = name + '[' + subName + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value !== undefined && value !== null)
                    {
                        query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
                    }
                }

                return query.length ? query.substr(0, query.length - 1) : query;
            };

            return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
        }];
})
    .config(function ($routeProvider) {
        $routeProvider
            .when('/', {
                templateUrl: 'views/main.html',
                controller: 'MainCtrl'
            })
            .otherwise({
                redirectTo: '/'
            });
    });

synopassApp.factory('Category', ['$http','$window','$route', function($http,$window,$route){
    var baseUrl = '../services/category';
    return{
        get: function(catId){
            return $http.get(baseUrl + '/' + catId);
        },
        query: function(){
            //2013-11-25 changed to post with master password for authentication

            /*
            return $http.get(baseUrl).then(function(result){
                return result.data;
            });
            */

            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });

            return $http({
                method: 'POST',
                url: baseUrl + "/list",
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    return response.data;
            });
        },
        delete: function(catId, catList, opt, xferCatId){
            var uri = baseUrl + "/" + catId + "/" + opt;

            if(opt == 3){
                uri += "/" + xferCatId;

            }else{
                uri += "/00000000-0000-0000-0000-000000000000";
            }


            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });

            return $http({
                method: 'DELETE',
                url: uri,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                var data = response.data;

                var resp = data.response;
                if(parseInt(resp) == 1){
                    //since no break provided by angularjs, a simple flag is used instead
                    var notBreak = true;
                    var idx = 0;
                    angular.forEach(catList, function(value, key){
                        if(notBreak && value.cat_id == catId){
                            catList.splice(idx, 1);
                            notBreak = false;
                        }
                        idx++;
                    });

                    return data;
                }else{
					if(data.err_code == $window.nonce_error){
						alert("Session expired. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.nonce_error;
					}else if(data.err_code == $window.error_max_login_failure){
						alert("Maximum consecutive login failure reached. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.error_max_login_failure;
					}else{
						alert("Failed to delete category, please try again in a minute.");
					}
                }
            });

        },
        save: function(category, catList, $scope){
            var is_add = category.id? false:true;

            if(is_add){
                var uri = baseUrl;
				
                var data = $.param({
                    json: JSON.stringify({
                        name: category.name,
                        p: $window.masterKey,
						n: $window.nonce
                    })
                });

                var result;

                return $http.post(uri, data).then(function(response){
                    var data = response.data;

                    var resp = data.response;
                    if(parseInt(resp) == 1){

                        var newobj = {
                            cat_id: data.cat_id,
                            cat_name: category.name
                        };

                        catList.push(newobj);

                        $scope.resetAddCatForm();
                    }else{
						if(data.err_code == $window.nonce_error){
							alert("Session expired. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.nonce_error;
						}else if(data.err_code == $window.error_max_login_failure){
							alert("Maximum consecutive login failure reached. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.error_max_login_failure;
							
						}else{
//                                          todo error popup
							alert("Failed to create category, please try again");
						}
                    }

                    return data.response;


                });

                var error;

                error = {response: 0, err_code: 99};
                return error;
            }else{

                var uri = baseUrl + "/" + category.id;

                var data = $.param({
                    json: JSON.stringify({
                        name: category.name,
                        p: $window.masterKey,
						n: $window.nonce
                    })
                });


                return $http({
                    method: 'PUT',
                    url: uri,
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                    data: data

                }).then(function(response){


                    var data = response.data;

                    var resp = data.response;
                    if(parseInt(resp) == 1){

                        //since no break provided by angularjs, a simple flag is used instead
                        var notBreak = true;
                        angular.forEach(catList, function(value, key){
                            if(notBreak && value.cat_id == category.id){
                                value.cat_name = category.name;
                                notBreak = false;
                            }
                        });

                    }else{
						if(data.err_code == $window.nonce_error){
							alert("Session expired. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.nonce_error;
						}else if(data.err_code == $window.error_max_login_failure){
							alert("Maximum consecutive login failure reached. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.error_max_login_failure;
						}else{
                            alert("Failed to update category, please try again.");
						}
                        
                    }

                    return data.response;


                });
            }
        }
    };
}]);

synopassApp.factory('PwdCache', function($cacheFactory){
    var pwdCache = $cacheFactory('pwdCache');

    return{
        set: function(id, pwdList){
            pwdCache.put(id, pwdList);
        },
        get: function(id){
            return pwdCache.get(id);
        },
        invalidate: function(id){
            pwdCache.remove(id);
        }
    };

});

synopassApp.factory('Password', ['$http', '$window', '$route', 'PwdCache', function($http, $window, $route, PwdCache){

    return{
        search: function(keyword,offset,pageSize){
            var baseUrl = '../services/password/search';

            var data = $.param({
                json: JSON.stringify({
                    kw: keyword,
                    p: $window.masterKey,
					n: $window.nonce
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

				var data = response.data;
				
				if(data.err_code == $window.nonce_error){
					alert("Session expired. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.nonce_error;
				}else if(data.err_code == $window.error_max_login_failure){
					alert("Maximum consecutive login failure reached. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.error_max_login_failure;
				}else{
					return data;
				}
            });
        },
        get: function(pwdId){
            var baseUrl = '../services/password';
            return $http.get(baseUrl + '/' + pwdId).then(function(result){
                return result.data;
            });
        },
        queryByCat: function(catId){


            var baseUrl = '../services/password/cat';

            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });

            return $http({
                method: 'POST',
                url: baseUrl + '/' + catId,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

				var data = response.data;
				
				if(data.err_code == $window.nonce_error){
					alert("Session expired. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.nonce_error;
				}else if(data.err_code == $window.error_max_login_failure){
					alert("Maximum consecutive login failure reached. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.error_max_login_failure;
				}else{
					return data;
				}
            });
        },
        queryByTag: function(tagId,offset,pageSize){
            var baseUrl = '../services/password/ctx';

            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });

            return $http({
                method: 'POST',
                url: baseUrl + '/' + tagId,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){
				var data = response.data;
				
				if(data.err_code == $window.nonce_error){
					alert("Session expired. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.nonce_error;
				}else if(data.err_code == $window.error_max_login_failure){
					alert("Maximum consecutive login failure reached. Please login again.");
					$window.masterKey = "";
					$window.nonce = "";
					$route.reload();
					return $window.error_max_login_failure;
				}else{
					return data;
				}
            });
        },
        delete: function(pwdId,pwdList){
            var baseUrl = '../services/password/' + pwdId;
            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });
            return $http({
                method: 'DELETE',
                url: baseUrl,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

            //return $http.delete(baseUrl).then(function(response){
                var data = response.data;

                var resp = data.response;

                if(parseInt(resp) == 1){
                    //since no break provided by angularjs, a simple flag is used instead
                    var notBreak = true;
                    var idx = 0;
                    angular.forEach(pwdList, function(value, key){
                        if(notBreak && value.pwd_id == pwdId){
                            pwdList.splice(idx, 1);
                            notBreak = false;
                        }
                        idx++;
                    });
                }else{
					if(data.err_code == $window.nonce_error){
						alert("Session expired. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.nonce_error;
					}else if(data.err_code == $window.error_max_login_failure){
						alert("Maximum consecutive login failure reached. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.error_max_login_failure;
					}else{
						alert("Failed to delete password, please try again.");
					}
                }
                return data;
            });
        },
        save: function(password, pwdList, ctxList,$scope, tagBtnList){
            var is_add = password.id? false:true;
            var baseUrl = '../services/password';

            if(is_add){

                var uri = baseUrl;

                var data = $.param({
                    json: JSON.stringify(password)
                });


                return $http({
                    method: 'POST',
                    url: uri,
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                    data: data

                }).then(function(response){


                        var data = response.data;

                        var resp = data.response;


                        if(parseInt(resp) == 1){

                            //password created


                            var new_pwd_id = data.pwd_id;

                            var tagList = "";

                            var isAddToCurrentTagList = false;


                            $("input[name=newTags]").each(function(){


                                var auri = baseUrl + "/" + new_pwd_id + "/context/name";
                                var tag_name = $(this).val();

                                var adata = $.param({
                                    json: JSON.stringify({
                                        ctx_name: tag_name,
                                        p: $window.masterKey,
										n: $window.nonce
                                    })
                                });

                                return $http({
                                    method: 'POST',
                                    url: auri,
                                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                                    data: adata

                                }).then(function(response){

                                        //tag added to db

                                        var data1 = response.data;

                                        var resp = data1.response;
                                        if(parseInt(resp) == 1){

                                            if(tagList != "")
                                                tagList += " ";
                                            tagList += tag_name;

                                            //add to tag list in case the tag is newly created

                                            //since no break provided by angularjs, a simple flag is used instead
                                            var notBreak = true;
                                            var isNewTag = true;

                                            angular.forEach(ctxList, function(value, key){
                                                if(notBreak && value.ctx_name == tag_name){
                                                    isNewTag = false;
                                                    notBreak = false;
                                                }
                                            });

                                            if(isNewTag){
                                                var newobj = {
                                                    ctx_id: data1.ctx_id,
                                                    ctx_name: tag_name
                                                };

                                                ctxList.push(newobj);

                                            }


                                            //adding new tag to password entry in password list.
                                            //for ajax async nature, the newly created password is most likely added to the password list below

                                            if($window.tabState == 0 && $window.currentCat == password.c){


                                                if(typeof pwdList != 'undefined'){

                                                    $.each(pwdList, function(i,val){

                                                        var curobj = val;
                                                        if(curobj.pwd_id == data.pwd_id){
                                                            curobj.ctx_name = tagList;
                                                            return false;
                                                        }
                                                    });
                                                }
                                            }


                                            if($window.tabState == 1 && $window.currentTagName == tag_name){
                                                var newtagobj = {
                                                    pwd_id: data.pwd_id,
                                                    pwd_pwd: password.p,
                                                    pwd_cat: password.c,
                                                    pwd_title: password.t,
                                                    pwd_login: password.l,
                                                    pwd_url: password.u,
                                                    pwd_notes: password.n
                                                };

                                                if(typeof pwdList != 'undefined')
                                                    pwdList.push(newtagobj);
                                                else
                                                    pwdList = newtagobj;
                                            }
                                        }else{
											if(data.err_code == $window.nonce_error){
												alert("Session expired. Please login again.");
												$window.masterKey = "";
												$window.nonce = "";
												$route.reload();
												return $window.nonce_error;
											}else if(data.err_code == $window.error_max_login_failure){
												alert("Maximum consecutive login failure reached. Please login again.");
												$window.masterKey = "";
												$window.nonce = "";
												$route.reload();
												return $window.error_max_login_failure;
											}else{
												alert("Failed to add tag to password, please try again.");
											}
                                        }

                                        if(isAddToCurrentTagList){
											//TODO
                                        }
                                        return data.response;


                                });
                            });


                            //check if current password list in the same category, push it then


                            if( $window.tabState == 0 && $window.currentCat == password.c){


                                var newobj = {
                                    pwd_id: data.pwd_id,
                                    pwd_pwd: password.p,
                                    pwd_cat: password.c,
                                    pwd_title: password.t,
                                    pwd_login: password.l,
                                    pwd_url: password.u,
                                    pwd_notes: password.n
                                };

                                if(typeof pwdList != 'undefined')
                                    pwdList.push(newobj);
                                else
                                    pwdList = newobj;
                            }

                            $scope.resetAddPwdForm(true);


                        }else{
							if(data.err_code == $window.nonce_error){
								alert("Session expired. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.nonce_error;
							}else if(data.err_code == $window.error_max_login_failure){
								alert("Maximum consecutive login failure reached. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.error_max_login_failure;
							}else{
								alert("Failed to add new password, please try again.");
							}
                        }

                        return data.response;


                });

            }else{          //begin edit mode

                //update password record

                var uri = baseUrl + "/" + password.id;

                var data = $.param({
                    json: JSON.stringify(password)
                });


                return $http({
                    method: 'PUT',
                    url: uri,
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                    data: data

                }).then(function(response){


                        var data = response.data;

                        var resp = data.response;
                        if(parseInt(resp) == 1){

                            //password record udpate success, update data in view

                            //since no break provided by angularjs, a simple flag is used instead
                            var notBreak = true;
                            angular.forEach(pwdList, function(value, key){
                                if(notBreak && value.pwd_id == password.id){
                                    value.pwd_pwd = password.p;
                                    value.pwd_cat = password.c;
                                    //value.pwd_icon = password.i;
                                    value.pwd_title = password.t;
                                    value.pwd_login = password.l;
                                    value.pwd_url = password.u;
                                    value.pwd_notes = password.o;
                                    notBreak = false;
                                }
                            });

                            //Now update the tags
                            var finaltag = "";

                            $("input[name=editTags]").each(function(){


                                var auri = baseUrl + "/" + password.id + "/context/name";
                                var tag_name = $(this).val();

                                if(finaltag != "")
                                    finaltag += " ";
                                finaltag += tag_name;

                                if(tag_name == '' || tag_name == " ")
                                    return true;

                                var adata = $.param({
                                    json: JSON.stringify({
                                        ctx_name: tag_name,
                                        p: $window.masterKey,
										n: $window.nonce
                                    })
                                });

                                return $http({
                                    method: 'POST',
                                    url: auri,
                                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                                    data: adata

                                }).then(function(response){


                                        var data1 = response.data;

                                        var resp = data1.response;
                                        if(parseInt(resp) == 1){
                                            //add to tag list in case the tag is newly created

                                            //since no break provided by angularjs, a simple flag is used instead
                                            var notBreak = true;
                                            var isNewTag = true;

                                            angular.forEach(ctxList, function(value, key){
                                                if(notBreak && value.ctx_name == tag_name){
                                                   isNewTag = false;
                                                    notBreak = false;
                                                    //invalidate cache
                                                    PwdCache.invalidate(value.ctx_id);
                                                }
                                            });

                                            ///push to tag list in view
                                            if(isNewTag){
                                                var newobj = {
                                                    ctx_id: data1.ctx_id,
                                                    ctx_name: tag_name
                                                };
                                                ctxList.push(newobj);

                                            }


                                            //add removal button for newly added tag, if not already existed
                                            var taglist = $('input#inputPwdContext').val();

                                            var taglistarray = taglist.split(" ");
                                            var isExisted = false;
                                            for(var j = 0; j < taglistarray.length; j++){
                                                if(taglistarray[j] == tag_name){
                                                    isExisted=true;
                                                    break;
                                                }
                                            }

                                            if(! isExisted){
                                                if($.trim(taglist) != "")
                                                    taglist += " ";
                                                taglist += tag_name;

                                                $('input#inputPwdContext').val(taglist);

                                                if(tagBtnList == null)
                                                    tagBtnList = tag_name;
                                                else
                                                    tagBtnList.push(tag_name);
                                            }


                                        }else{
											if(data.err_code == $window.nonce_error){
												alert("Session expired. Please login again.");
												$window.masterKey = "";
												$window.nonce = "";
												$route.reload();
												return $window.nonce_error;
											}else if(data.err_code == $window.error_max_login_failure){
												alert("Maximum consecutive login failure reached. Please login again.");
												$window.masterKey = "";
												$window.nonce = "";
												$route.reload();
												return $window.error_max_login_failure;
											}else{
												alert("Failed to add tag to password, please try again.");
											}
                                        }


                                        return data.response;


                                });
                            });     //end loop of input['editTags']


                            var endlist = $('input#inputPwdContext').val();

                            if($.trim(endlist) != "" && $.trim(finaltag) != "")
                                endlist += " ";
                            $scope.pwdObj.ctx_name=endlist + finaltag;

                            $('input#inputPwdContext').val(endlist + finaltag)



                        }else{
                            if(data.err_code == $window.nonce_error){
								alert("Session expired. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.nonce_error;
							}else if(data.err_code == $window.error_max_login_failure){
								alert("Maximum consecutive login failure reached. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.error_max_login_failure;
							}else{
								alert("Failed to update password, please try again.");
							}
                        }

                        return data.response;


                });
            }
        }
    };
}]);


synopassApp.factory('Context', ['$http','$window', '$route',  function($http, $window, $route, TagCache){
    var baseUrl = '../services/context';
    return{
        get: function(ctxId){
            //return $http.get(baseUrl + '/' + ctxId);
            var pwdList = TagCache.get(ctxId);
            if(! pwdList){
                console.log('Found from cache, return from cache instead.');
                return pwdList;
            }
           // console.log('Cache not found, return from web service.');
            pwdList = $http.get(baseUrl + '/' + ctxId);
            TagCache.set(ctxId, pwdList);
            return pwdList;
        },
        query: function(){
            /*
            return $http.get(baseUrl).then(function(result){
                return result.data;
            });           */

            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });

            return $http({
                method: 'POST',
                url: baseUrl + "/list",
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    return response.data;
            });
        },
        delete: function(ctxId, ctxList, pwdList){
            var uri = baseUrl + "/" + ctxId;
            var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });
            return $http({
                method: 'DELETE',
                url: uri,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){
            //return $http.delete(uri).then(function(response){
                var data = response.data;

                var resp = data.response;
                if(parseInt(resp) == 1){
                    //since no break provided by angularjs, a simple flag is used instead
                    var notBreak = true;
                    var idx = 0;
                    angular.forEach(ctxList, function(value, key){
                        if(notBreak && value.ctx_id == ctxId){
                            ctxList.splice(idx, 1);
                            notBreak = false;
                        }
                        idx++;
                    });

                    //from the tag from all password in the curren tpassword view
                    //since the password list is not cached yet, so only current view need to be processed atm

                    angular.forEach(pwdList, function(value, idx){
                        var tags = value.ctx_name;

                        if(tags != null && $.trim(tags) != ""){
                            var tagslist = tags.split(" ");
                            var newtags = "";
                            for(var j = 0; j < tagslist.length; j++){
                                if(tagslist[j] != $('span#delTagTitle').html()){
                                    if($.trim(newtags) != "")
                                        newtags += " ";
                                    newtags +=  tagslist[j];
                                }
                            }
                            value.ctx_name = newtags;
                        }

                    });

                    return data;
                }else{
					if(data.err_code == $window.nonce_error){
						alert("Session expired. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.nonce_error;
					}else if(data.err_code == $window.error_max_login_failure){
						alert("Maximum consecutive login failure reached. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.error_max_login_failure;
					}else{
//                                         todo error popup
						alert("Failed to delete context, please try again in a minute.");
					}
                    
                }
            });

        },
        save: function(context, ctxList, $scope){
            var is_add = context.id? false:true;

            if(is_add){
                var uri = baseUrl;

                var data = $.param({
                    json: JSON.stringify({
                        name: context.name,
						p: $window.masterKey,
						n: $window.nonce
                    })
                });

                var result;

                return $http.post(uri, data).then(function(response){
                    var data = response.data;

                    var resp = data.response;
                    if(parseInt(resp) == 1){

                        var newobj = {
                            ctx_id: data.ctx_id,
                            ctx_name: context.name
                        };

                        ctxList.push(newobj);

                        $scope.resetAddTagForm();
                    }else{
						if(data.err_code == $window.nonce_error){
							alert("Session expired. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.nonce_error;
						}else if(data.err_code == $window.error_max_login_failure){
							alert("Session exMaximum consecutive login failure reached. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.error_max_login_failure;
						}else{
//                                          todo error popup
							alert("Failed to create context, please try again");
						}
                        
                    }

                    return data.response;


                });

                var error;

                error = {response: 0, err_code: 99};
                return error;
            }else{

                var uri = baseUrl + "/" + context.id;

                var data = $.param({
                    json: JSON.stringify({
                        name: context.name,
                        p: $window.masterKey,
						n: $window.nonce
                    })
                });


                return $http({
                    method: 'PUT',
                    url: uri,
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                    data: data

                }).then(function(response){


                        var data = response.data;

                        var resp = data.response;
                        if(parseInt(resp) == 1){

                            //since no break provided by angularjs, a simple flag is used instead
                            var notBreak = true;
                            angular.forEach(ctxList, function(value, key){
                                if(notBreak && value.ctx_id == context.id){
                                    value.ctx_name = context.name;
                                    notBreak = false;
                                }
                            });

                        }else{
							if(data.err_code == $window.nonce_error){
								alert("Session expired. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.nonce_error;
							}else if(data.err_code == $window.error_max_login_failure){
								alert("Maximum consecutive login failure reached. Please login again.");
								$window.masterKey = "";
								$window.nonce = "";
								$route.reload();
								return $window.error_max_login_failure;
							}else{
//                                          todo error popup
								alert("Failed to update context, please try again.");
							}
                        }

                        return data.response;


                    });
            }
        }
    };
}]);


synopassApp.factory('PasswordContext', ['$http', '$window', '$route', function($http, $window, $route){
    var baseUrl = '../services/password';
    return{

        delete: function(pwdId, ctxName,tagList){
            var uri = baseUrl + "/" + pwdId + "/context/name/" + ctxName;
			
			var data = $.param({
                json: JSON.stringify({
                    p: $window.masterKey,
					n: $window.nonce
                })
            });
            return $http({
                method: 'DELETE',
                url: uri,
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){
            //return $http.delete(uri).then(function(response){
                var data = response.data;

                var resp = data.response;
                if(parseInt(resp) == 1){
                    var idx = tagList.indexOf(ctxName);
                    if(idx > -1){
                        tagList.splice(idx,1);
                        var newTagIdx = tagList.toString().replace(/,+/g," ");

                        $("input#inputPwdContext").val(newTagIdx);
                    }

                    return data;
                }else{
					if(data.err_code == $window.nonce_error){
						alert("Session expired. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.nonce_error;
					}else if(data.err_code == $window.error_max_login_failure){
						alert("Maximum consecutive login failure reached. Please login again.");
						$window.masterKey = "";
						$window.nonce = "";
						$route.reload();
						return $window.error_max_login_failure;
					}else{
						alert("Failed to delete context, please try again in a minute.");
					}
                }
            });

        },
        add: function(pwdId, ctxName){

                var uri = baseUrl + "/" + pwdId + "/context/name/" + ctxName;
                var uri = baseUrl;

                var data = $.param({
                    json: JSON.stringify({
                        name: context.name
                    })
                });

                var result;

                return $http.post(uri, data).then(function(response){
                    var data = response.data;

                    var resp = data.response;
                    if(parseInt(resp) == 1){

                        var newobj = {
                            ctx_id: data.ctx_id,
                            ctx_name: context.name
                        };

                        ctxList.push(newobj);

                    }else{
                        alert("Failed to create context, please try again");
                    }

                    return data.response;


                });

                var error;

                error = {response: 0, err_code: 99};
                return error;
        }


    };
}]);







synopassApp.factory('Config', ['$http', function($http){
    var baseUrl = '../services/setup';
    return{
        isInited: function(){
            return $http.get(baseUrl + '/isinited');
        },
        save: function(master,enckey){

            var data = $.param({
                json: JSON.stringify({
                    k: enckey,
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/save',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                var resp = response.data;

                if(resp.response == 1){
                    //successfully saved
                    alert('Configuration saved, please login to continue.');
                    $('div#setup').modal('hide');
                    $('div#login').modal('show');
                }else{
                    alert('failed to save configuration, please try again later.');
                }
                return response.data;
            });
        },
		is2factor: function(){
            return $http.get(baseUrl + '/is_2_factor');
        },
    };
}]);





synopassApp.factory('Auth', ['$http', '$window', function($http,$window){
    var baseUrl = '../services';
    return{
        login: function(master){

            var data = $.param({
                json: JSON.stringify({
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/login',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Login success.');
                        $window.masterKey = master;
						$window.nonce = resp.nonce;
                        $('div#login').modal('hide');
					}else if(resp.err_code == $window.error_max_login_failure){
							alert("Maximum consecutive login failure reached. Please try again later.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.error_max_login_failure;
                    }else{
                        alert('Login failed, please try again later.');
                    }
                    return response.data;
                });
        },
		two_factor: function(master,code){

            var data = $.param({
                json: JSON.stringify({
                    p: master,
					c: code
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/login_two_factor',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Login success.');
                        $window.masterKey = master;
						$window.nonce = resp.nonce;
                        $('div#login').modal('hide');
					}else if(resp.err_code == $window.error_max_login_failure){
							alert("Maximum consecutive login failure reached. Please try again later.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.error_max_login_failure;
                    }else{
                        alert('Login failed, please try again later.');
                    }
                    return response.data;
                });
        }
    };
}]);


