'use strict';

var twoFactorAuthApp = angular.module('setup2FactorAuthApp', [],function($httpProvider){

        // Use x-www-form-urlencoded Content-Type
        $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

        // Override $http service's default transformRequest
        $httpProvider.defaults.transformRequest = [function(data)
        {
            /**
             * The workhorse; converts an object to x-www-form-urlencoded serialization.
             * @param {Object} obj
             * @return {String}
             */
            var param = function(obj)
            {
                var query = '';
                var name, value, fullSubName, subName, subValue, innerObj, i;

                for(name in obj)
                {
                    value = obj[name];

                    if(value instanceof Array)
                    {
                        for(i=0; i<value.length; ++i)
                        {
                            subValue = value[i];
                            fullSubName = name + '[' + i + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value instanceof Object)
                    {
                        for(subName in value)
                        {
                            subValue = value[subName];
                            fullSubName = name + '[' + subName + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value !== undefined && value !== null)
                    {
                        query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
                    }
                }

                return query.length ? query.substr(0, query.length - 1) : query;
            };

            return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
        }];
})
    .config(function ($routeProvider) {
        $routeProvider
            .when('/', {
                templateUrl: 'views/2FactorAuth/main.html',
                controller: 'MainCtrl'
            })
            .otherwise({
                redirectTo: '/'
            });
    });
	
	
twoFactorAuthApp.factory('Auth', ['$http', '$window', function($http,$window){
    var baseUrl = '../services';
    return{
        login: function(master){

            var data = $.param({
                json: JSON.stringify({
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/login',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Login success.');
                        $window.masterKey = master;
                        $('div#login').modal('hide');
                    }else{
                        alert('Login failed, please try again later.');
                    }
                    return response.data;
                });
        }
    };
}]);

twoFactorAuthApp.factory('Salt', ['$http', '$window', function($http,$window){
    var baseUrl = '../services';
    return{
        generate: function(master){

            var data = $.param({
                json: JSON.stringify({
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/two_factor_key/generate',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
						//display qr code
						
						$('div#fail').hide();
						$('#btn-fail').hide();
						
						
						
						$('img#raw_qrcode').attr("src","http://www.google.com/chart?chs=200x200&chld=M|0&cht=qr&chl=otpauth://totp/Synopass?secret=" + resp.key);
						
						$('#btn-success').show();
						$('div#qrcode').show();
						return true;
                    }else{
						$('#btn-success').hide();
						$('div#qrcode').hide();
						
						$('#btn-fail').show();
						$('div#fail').show();

						return false;
                    }
                });
        }, 
		disable: function(master){

            var data = $.param({
                json: JSON.stringify({
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/two_factor_key/disable',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

					$('div#fail').hide();
					$('#btn-fail').hide();
					$('#btn-success').hide();
					$('div#qrcode').hide();

                    var resp = response.data;

                    if(resp.response == 1){
						alert("2 Factor Authentication disabled");
						return true;
                    }else{
						alert("2 Factor Authentication failed, please try again.");
						return false;
                    }
                });
        }
    };
}]);