'use strict';

var setupApp = angular.module('setupApp', [],function($httpProvider){

        // Use x-www-form-urlencoded Content-Type
        $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

        // Override $http service's default transformRequest
        $httpProvider.defaults.transformRequest = [function(data)
        {
            /**
             * The workhorse; converts an object to x-www-form-urlencoded serialization.
             * @param {Object} obj
             * @return {String}
             */
            var param = function(obj)
            {
                var query = '';
                var name, value, fullSubName, subName, subValue, innerObj, i;

                for(name in obj)
                {
                    value = obj[name];

                    if(value instanceof Array)
                    {
                        for(i=0; i<value.length; ++i)
                        {
                            subValue = value[i];
                            fullSubName = name + '[' + i + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value instanceof Object)
                    {
                        for(subName in value)
                        {
                            subValue = value[subName];
                            fullSubName = name + '[' + subName + ']';
                            innerObj = {};
                            innerObj[fullSubName] = subValue;
                            query += param(innerObj) + '&';
                        }
                    }
                    else if(value !== undefined && value !== null)
                    {
                        query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
                    }
                }

                return query.length ? query.substr(0, query.length - 1) : query;
            };

            return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
        }];
})
    .config(function ($routeProvider) {
        $routeProvider
            .when('/', {
                templateUrl: 'views/setup/main.html',
                controller: 'MainCtrl'
            })
            .otherwise({
                redirectTo: '/'
            });
    });
	
	
setupApp.factory('Auth', ['$http', '$window', function($http,$window){
    var baseUrl = '../services';
    return{
        login: function(master){

            var data = $.param({
                json: JSON.stringify({
                    p: master
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/login',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Login success.');
                        $window.masterKey = master;
						$window.nonce = resp.nonce;
						
                        $('div#login').modal('hide');
                    }else{
                        alert('Login failed, please try again later.');
                    }
                    return response.data;
                });
        },
		two_factor: function(master,code){

            var data = $.param({
                json: JSON.stringify({
                    p: master,
					c: code
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/login_two_factor',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Login success.');
                        $window.masterKey = master;
						$window.nonce = resp.nonce;
						
                        $('div#login').modal('hide');
                    }else{
                        alert('Login failed, please try again later.');
                    }
                    return response.data;
                });
        }
    };
}]);

setupApp.factory('Config', ['$http', '$window', '$route', function($http,$window,$route){
    var baseUrl = '../services/setup';
    return{
        update_gcm: function(master,nonce,enable,senderId,apiKey,isLockRegId,isEmail,email){
		
            var data = $.param({
                json: JSON.stringify({
                    p: master,
					n: nonce,
					e: enable,
					s: senderId,
					k: apiKey,
					l: isLockRegId,
					a: isEmail,
					b: email
                })
            });


            return $http({
                method: 'POST',
                url: baseUrl + '/gcm/save',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data

            }).then(function(response){

                    var resp = response.data;

                    if(resp.response == 1){
                        //successfully saved
                        alert('Config saved.');
						return resp;
                    }else{
						if(resp.err_code == $window.nonce_error){
							alert("Session expired. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.nonce_error;
						}else{
							alert('Saving GCM setting failed, please try again later.');
						}
                    }
                    return resp;
              });
        },
		is2factor: function(){
            return $http.get(baseUrl + '/is_2_factor');
        },
		get: function(master,nonce){
           var data = $.param({
                json: JSON.stringify({
                    p: master,
					n: nonce
                })
            });

            return $http({
                method: 'POST',
                url: baseUrl + '/get',
                headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                data: data
            }).then(function(response){
                    var resp = response.data;

					if(resp.err_code == $window.nonce_error){
							alert("Session expired. Please login again.");
							$window.masterKey = "";
							$window.nonce = "";
							$route.reload();
							return $window.nonce_error;
					}
					
					$window.config = resp.data;
					return resp.data;					
              });
        }
    };
}]);