SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

CREATE SCHEMA IF NOT EXISTS `synopass` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci ;
SHOW WARNINGS;
USE `synopass` ;

-- -----------------------------------------------------
-- Table `synopass`.`Category`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Category` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Category` (
  `cat_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `cat_name` VARCHAR(200) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  PRIMARY KEY (`cat_id`) )
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Icon`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Icon` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Icon` (
  `icon_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `icon_filename` VARCHAR(45) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  PRIMARY KEY (`icon_id`) )
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Context`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Context` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Context` (
  `ctx_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `ctx_name` VARCHAR(200) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  PRIMARY KEY (`ctx_id`) )
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Password`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Password` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Password` (
  `pwd_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_pwd` VARCHAR(100) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_cat` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_icon` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NULL ,
  `pwd_title` VARCHAR(200) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_login` VARCHAR(100) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_url` VARCHAR(1000) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NULL ,
  `pwd_notes` TEXT CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NULL ,
  PRIMARY KEY (`pwd_id`) ,
  INDEX `fk_Password_Category_idx` (`pwd_cat` ASC) ,
  INDEX `fk_Password_Icon1_idx` (`pwd_icon` ASC) ,
  CONSTRAINT `fk_Password_Category`
    FOREIGN KEY (`pwd_cat` )
    REFERENCES `synopass`.`Category` (`cat_id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_Password_Icon1`
    FOREIGN KEY (`pwd_icon` )
    REFERENCES `synopass`.`Icon` (`icon_id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Password_Change`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Password_Change` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Password_Change` (
  `pwd_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_old_pwd` VARCHAR(100) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_new_pwd` VARCHAR(100) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_old_notes` TEXT CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NULL ,
  `pwd_new_notes` TEXT CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NULL ,
  PRIMARY KEY (`pwd_id`) ,
  CONSTRAINT `fk_Password_Change_Password`
    FOREIGN KEY (`pwd_id` )
    REFERENCES `synopass`.`Password` (`pwd_id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Password_Context`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Password_Context` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Password_Context` (
  `pc_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `pwd_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `ctx_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  PRIMARY KEY (`pc_id`) ,
  INDEX `fk_Password_Context_Password1_idx` (`pwd_id` ASC) ,
  INDEX `fk_Password_Context_Context1_idx` (`ctx_id` ASC) ,
  CONSTRAINT `fk_Password_Context_Password1`
    FOREIGN KEY (`pwd_id` )
    REFERENCES `synopass`.`Password` (`pwd_id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_Password_Context_Context1`
    FOREIGN KEY (`ctx_id` )
    REFERENCES `synopass`.`Context` (`ctx_id` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

SHOW WARNINGS;

-- -----------------------------------------------------
-- Table `synopass`.`Config`
-- -----------------------------------------------------
-- DROP TABLE IF EXISTS `synopass`.`Config` ;

SHOW WARNINGS;
CREATE  TABLE IF NOT EXISTS `synopass`.`Config` (
  `config_id` VARCHAR(40) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `config_key` VARCHAR(100) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  `config_value` VARCHAR(1000) CHARACTER SET 'utf8' COLLATE 'utf8_general_ci' NOT NULL ,
  PRIMARY KEY (`config_id`) ,
  UNIQUE INDEX `id_UNIQUE` (`config_id` ASC) ,
  UNIQUE INDEX `key_UNIQUE` (`config_key` ASC) )
ENGINE = InnoDB;

SHOW WARNINGS;
USE `synopass` ;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;

-- -------------------------------
-- initialize data
-- -------------------------------
insert into Category (cat_id,cat_name) values ('00000000-0000-0000-0000-000000000000', 'Uncategorized');