<?php

define('DEST_LOGFILE', '/var/services/web/synopass/services/php.log');
ini_set("error_log", DEST_LOGFILE);

require 'Slim/Slim.php';

\Slim\Slim::registerAutoloader();

$app = new \Slim\Slim();

require('../library/sanitize.inc.php');

require('../classes/base_service.class.php');
require('../classes/password.class.php');
require('../classes/category.class.php');
require('../classes/context.class.php');
require('../classes/password_context.class.php');
require('../classes/config.class.php');
require('../classes/auth.class.php');
require('../classes/ga.class.php');
require('../classes/gcm.class.php');

//Begin synopass RESTful services

require('inc/Password/password_list.inc.php');
require('inc/Password/password_list_by_cat.inc.php');
require('inc/Password/password_list_by_ctx.inc.php');
require('inc/Password/password_view.inc.php');
require('inc/Password/password_add.inc.php');
require('inc/Password/password_update.inc.php');
require('inc/Password/password_delete.inc.php');
require('inc/Password/password_search.inc.php');

require('inc/Category/category_list.inc.php');
require('inc/Category/category_update.inc.php');
require('inc/Category/category_delete.inc.php');
require('inc/Category/category_add.inc.php');
require('inc/Category/category_view.inc.php');


require('inc/Context/context_add.inc.php');
require('inc/Context/context_update.inc.php');
require('inc/Context/context_delete.inc.php');
require('inc/Context/context_list.inc.php');
require('inc/Context/context_view.inc.php');

require('inc/Password_Context/password_context_add.inc.php');
require('inc/Password_Context/password_context_add_by_name.inc.php');
require('inc/Password_Context/password_context_list.inc.php');
require('inc/Password_Context/password_context_delete.inc.php');
require('inc/Password_Context/password_context_delete_by_name.inc.php');
require('inc/Password_Context/password_context_delete_all.inc.php');

require('inc/Config/setup_is_inited.inc.php');
require('inc/Config/setup_is_2factor.inc.php');
require('inc/Config/setup_save.inc.php');
require('inc/Config/update_master_pwd.inc.php');
require('inc/Config/setup_gcm.inc.php');
require('inc/Config/setup_get.inc.php');
require('inc/Config/setup_gcm_reg.inc.php');

require('inc/Security/sec_login.inc.php');
require('inc/Security/sec_2_factor.inc.php');
require('inc/Security/sec_login_2_factor.inc.php');
require('inc/Security/sec_2_factor_disable.inc.php');

$app->run();

function check_json($json){
	if(empty($json)){
		missing_parameter();
		die;
	}
}
function missing_parameter(){
	$error = array('response' => 0, 'err_code' => 1, 'err_desc' => 'Missing parameter');

	echo json_encode($error);
}
function parse_result($result){
	if($result === 11)
		$error = array('response' => 0, 'err_code' => 11, 'err_desc' => 'Database connection failed.');
	else
		$error = $result;
	
	echo json_encode($error);
}

function get_encryption_key($master){
	if(empty($master))
		return null;
		
	$config = new Config();
	return $config->get_encryption_key($master);
}

/* essential parameter: p, n 

	return true: success
	return 100: nonce fail
	return json: login fail, terminate
*/
function login($param,$pname){
	
	
	
	foreach ($pname as &$value) {
		if(empty($param->{$value})){
			missing_parameter();
			die;
		}
	}

	$login_count = block_failed_login();
	
    $auth = new Auth();
    $login = $auth->login($param->{'p'});
    if($login === true){
	
		$auth->reset_fail_count();
		$nonce = $auth->verify_nonce($param->{'n'});

		if($nonce === true){
			return true;
		}else{
			return 100;
		}
    }else{
		increment_fail_login($login_count);
        $error = array('response' => 0, 'err_code' => $result, 'err_desc' => 'login failed');
        echo json_encode($error);
		die;
	}
}

function push_password_view($enc_key, $message){
	$config = new Config();
	$is_gcm_enabled = $config->is_gcm_enabled();
	$count_reg_id = $config->count_reg_id();
	
	if($is_gcm_enabled && $count_reg_id > 0){
	
		$gcm = new GCM();
			
		$key = $config->get_gcm_api_key($enc_key);
		$regId = $config->get_gcm_first_reg_id($enc_key);

		$msg = array("msg" => $message);
 
		$gcm->send_notification(array($regId),$key,$msg);
	}
}

function block_failed_login(){
	
	$auth = new Auth();
	$count = $auth->get_fail_count();
	
	if($count > 5){
		$error = array('response' => 0, 'err_code' => 101, 'err_desc' => 'Max consecutive login failed reached');
        echo json_encode($error);
		die;
	}
	
	return $count;
}

function increment_fail_login($current_fail_count){
	$current_fail_count++;
	
	$auth = new Auth();
	$auth->increment_fail($current_fail_count);
}